@@ -7,23 +7,24 @@ Contains:
 import numpy as np
 import pandas_market_calendars as mcal
 
-def NYSEgen(sdate = np.datetime64('1980-01-01'),
-            edate = np.datetime64('2050-12-31')):
+def NYSEgen(sdate='1980-01-01', edate='2050-12-31'):
     """
     Generate `numpy.busdaycalendar` for NYSE between `sdate` and 
     `edate`.
 
     Parameters
     ----------
-    `sdate` : `numpy.datetime64`, optional
-        Calendar start date. The default is `numpy.datetime64('1980-01-01')`.
-    `edate` : numpy.datetime64, optional
-        Calendar end date. The default is `numpy.datetime64('2050-12-31')`.
+    `sdate` : str date, optional
+        Calendar start date. The default is `'1980-01-01'`.
+    `edate` : str date, optional
+        Calendar end date. The default is `'2050-12-31'`.
     Returns
     -------
     `numpy.busdaycalendar`
         NYSE business calendar.
     """
+    sdate_ = np.datetime64(sdate)
+    edate_ = np.datetime64(edate)
     # get the NYSE holiday list from standard pandas_market_calendars
     ldates = list(mcal.get_calendar('NYSE').holidays().holidays)
     # append here additional holidays 
@@ -34,5 +35,5 @@ def NYSEgen(sdate = np.datetime64('1980-01-01'),
     ldates.sort()
     # move to np.array and return the business calendar
     hdates = np.array(ldates)
-    hdates = hdates[(hdates >= sdate) & ( hdates <= edate)]
+    hdates = hdates[(hdates >= sdate_) & ( hdates <= edate_)]
     return np.busdaycalendar(holidays=hdates)
\ No newline at end of file
