@@ -163,7 +163,8 @@ class BTSDAnalyzer(BTADAnalyzer):
         
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['infostring']}")
+            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
+                        + f"on calibration date {self.rrate.index[-1]}")
             return np.array([np.nan] * mm)
             
         # mBTSD
@@ -248,7 +249,8 @@ class BTSDAnalyzer(BTADAnalyzer):
         
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']}")
+            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
+                        + f"on calibration date {self.rrate.index[-1]}")
             return np.array([np.nan] * mm)
             
         t = res['x'][-1]
@@ -340,7 +342,8 @@ class BTSDAnalyzer(BTADAnalyzer):
         
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']}")
+            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
+                        + f"on calibration date {self.rrate.index[-1]}")
             return np.array([np.nan] * mm)
             
         t = res['x'][-1]
@@ -430,7 +433,8 @@ class BTSDAnalyzer(BTADAnalyzer):
         
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']}")
+            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
+                        + f"on calibration date {self.rrate.index[-1]}")
             return np.array([np.nan] * mm)
             
         # rate of return
@@ -512,7 +516,8 @@ class BTSDAnalyzer(BTADAnalyzer):
         
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']}")
+            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
+                        + f"on calibration date {self.rrate.index[-1]}")
             return np.array([np.nan] * mm)
             
         # optimal weights
@@ -529,4 +534,101 @@ class BTSDAnalyzer(BTADAnalyzer):
         self.secondary_risk_comp = self.alpha.copy()
         
         return self.ww
+    
+    
+    def _risk_diversification(self, d=1):
+        # Order of variables:
+        # w <- [0 : mm]
+        # then for l <- [0:ll]
+        #   u_l <- mm + l(nn+1), 
+        #   s_l <- [mm + l(nn + 1) + 1: mm + (l + 1)(nn + 1)]
+        # t <- mm + ll * (nn + 1)
+        # in total dim = mm + ll(nn + 1) + 1
+        nn = self.nn
+        mm = self.mm
+        ll = self.ll
+        
+        # build c
+        c_data = [0.] * mm 
+        for l in range(ll):
+            c_data += [self.coef[l]] + [0.] * nn
+        c_data += [0.]
+        
+        
+        # build G
+        # linear
+        G_icol = list(range(mm)) * (nn * ll)
+        G_irow = [k  for k in range(nn * ll) for _ in range(mm)]
+        G_data = list(np.ravel(-self.rrate)) * ll
+        for l in range(ll):
+            G_icol += [mm + ll * (nn + 1)] * nn \
+                  + list(range(mm + l * (nn + 1) + 1, mm + (l + 1) * (nn + 1)))
+            G_irow += list(range(l * nn, (l + 1) * nn)) * 2
+            G_data += [self.alpha[l]] * nn + [-1.] * nn 
+            
+        G_icol += list(range(mm)) + [mm + ll * (nn + 1)]
+        G_irow += [nn * ll] * (mm + 1)
+        G_data += list(-self.muk * d) + [self.mu * d]
+            
+        G_icol += list(range(mm + ll * (nn + 1) + 1))
+        G_irow += list(range(nn * ll + 1, 1 + mm + ll * (2 * nn + 1) + 1))
+        G_data += [-1.] * (mm + ll * (nn + 1) + 1)
+        # cone
+        for l in range(ll):
+            G_icol += list(range(mm + l * (nn + 1), mm + (l + 1) * (nn + 1)))
+            G_irow += \
+                list(range(2 + mm + ll * (2 * nn + 1) + l * (nn + 1), 
+                     2 + mm + ll * (2 * nn + 1) + (l + 1) * (nn + 1)))
+            G_data += [-np.sqrt(nn)] + [-1.] * nn
+            
+        G_shape = (2 + mm + ll * (3* nn + 2), mm + ll * (nn + 1) + 1)
+        G = sps.coo_matrix((G_data, (G_irow, G_icol)), G_shape)
+        
+        # build h
+        h_data = [0.] * (2 + mm + ll * (3 * nn + 2))
+        
+        # build dims
+        dims = {'l': (2 + mm + ll * (2 * nn + 1)), 'q': [nn + 1] * ll}
+        
+        # build A
+        A_icol = list(range(mm)) + [mm + ll * (nn + 1)]
+        A_irow = [0] * (mm + 1)
+        A_data = [1.] * mm + [-1.]
+        
+        A_icol += list(range(mm))
+        A_irow += [1] * mm
+        A_data += list(self.risk_comp)
+
+        A_shape = (2, mm + ll * (nn + 1) + 1)
+        A = sps.coo_matrix((A_data, (A_irow, A_icol)), A_shape)
+        
+        # build b
+        b_data = [0., 1.]
+        
+        # calc
+        res = _socp_solver(self.method, c_data, G, h_data, dims, A, b_data)
+        
+        self.status = res['status']
+        if self.status != 0:
+            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
+                        + f"on calibration date {self.rrate.index[-1]}")
+            return np.array([np.nan] * mm)
+            
+        t = res['x'][-1]
+        # BTSD-Divers
+        self.divers = 1. - res['pcost']
+        # mBTSD
+        self.risk =  res['pcost'] / t
+        # optimal weights
+        self.ww = np.array(res['x'][:mm] / t)
+        self.ww.shape = mm
+        # rate of return
+        self.RR = np.dot(self.ww, self.muk)
+        # BTSD components
+        self.primary_risk_comp = \
+            np.array([res['x'][mm + l * (nn + 1)] / t for l in range(ll)])
+        # BTSD thresholds
+        self.secondary_risk_comp = self.alpha.copy()
+        
+        return self.ww
     
\ No newline at end of file
