@@ -165,7 +165,8 @@ class GINIAnalyzer(_RiskAnalyzer):
  
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']}")
+            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
+                        + f"on calibration date {self.rrate.index[-1]}")
             return np.array([np.nan] * mm)
             
         # GINI
@@ -231,7 +232,8 @@ class GINIAnalyzer(_RiskAnalyzer):
  
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']}")
+            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
+                        + f"on calibration date {self.rrate.index[-1]}")
             return np.array([np.nan] * mm)
             
         t = res['x'][-1]
@@ -300,7 +302,8 @@ class GINIAnalyzer(_RiskAnalyzer):
  
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']}")
+            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
+                        + f"on calibration date {self.rrate.index[-1]}")
             return np.array([np.nan] * mm)
             
         t = res['x'][-1]
@@ -368,7 +371,8 @@ class GINIAnalyzer(_RiskAnalyzer):
  
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']}")
+            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
+                        + f"on calibration date {self.rrate.index[-1]}")
             return np.array([np.nan] * mm)
             
         # optimal weights
@@ -431,7 +435,8 @@ class GINIAnalyzer(_RiskAnalyzer):
  
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"Warning {res['status']}: {res['infostring']}")
+            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
+                        + f"on calibration date {self.rrate.index[-1]}")
             return np.array([np.nan] * mm)
             
         # optimal weights
@@ -446,4 +451,82 @@ class GINIAnalyzer(_RiskAnalyzer):
         self.secondary_risk_comp = np.array([self.risk])
         
         return self.ww
+    
+    
+    def _risk_diversification(self, d=1):
+        # Order of variables (mm - no of symb, nn - no of observations)
+        # w <- [0 : mm]
+        # s <- [mm : mm + nn2]
+        # t <- mm + nn2
+        # in total dim = mm + nn2 + 1
+        # where nn2 = nn * (nn - 1) / 2
+        mm = self.mm
+        nn2 = self.nn2
+        norm = 1 / self.nn ** 2
+        
+        # build c
+        c_data = [0.] * mm + [norm] * nn2 + [0.]
+        
+        # build G
+        G_icol = [m for m in range(mm) for _ in range(nn2)] * 2
+        G_irow = list(range(nn2)) * mm + list(range(nn2, 2 * nn2)) * mm
+        G_data = list(self.drate) + list(-self.drate)
+        
+        G_icol += list(range(mm, mm + nn2)) * 2
+        G_irow += list(range(nn2 * 2))
+        G_data += [-1.] * (nn2 * 2)
+        
+        G_icol += list(range(mm)) + [mm + nn2]
+        G_irow += [nn2 * 2] * (mm + 1)
+        G_data += list(-self.muk * d) + [self.mu * d]
+        
+        G_icol += list(range(mm))
+        G_irow += list(range(nn2 * 2 + 1, nn2 * 2 + mm + 1))
+        G_data += [-1.] * mm
+        
+        G_shape = (nn2 * 2 + mm + 1, mm + nn2 + 1)
+        G = sps.coo_matrix((G_data, (G_irow, G_icol)), G_shape)
+        
+        # build h
+        h_data = [0.] * (nn2 * 2 + mm + 1) 
+        
+        # build A
+        A_icol = list(range(mm))
+        A_irow = [0] * mm
+        A_data = list(self.risk_comp)
+        
+        A_icol += list(range(mm)) + [mm + nn2]
+        A_irow += [1] * (mm + 1)
+        A_data += [1.] * mm + [-1.]
+        
+        A_shape = (2, mm + nn2 + 1)
+        A = sps.coo_matrix((A_data, (A_irow, A_icol)), A_shape)
+        
+        # build b
+        b_data = [1., 0.]
+        
+        # calc
+        res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
+ 
+        self.status = res['status']
+        if self.status != 0:
+            warnings.warn(f"Warning {res['status']}: {res['infostring']} "
+                        + f"on calibration date {self.rrate.index[-1]}")
+            return np.array([np.nan] * mm)
+            
+        t = res['x'][-1]
+        # Gini-Divers
+        self.divers = 1. - res['pcost']
+        # Gini
+        self.risk = res['pcost'] / t
+        # optimal weights
+        self.ww = np.array(res['x'][:mm] / t)
+        self.ww.shape = mm
+        # rate of return
+        self.RR = np.dot(self.ww, self.muk)
+        # default to risk
+        self.primary_risk_comp = np.array([self.risk])
+        self.secondary_risk_comp = np.array([self.risk])
+        
+        return self.ww
         
\ No newline at end of file
