@@ -3,6 +3,7 @@ import numpy as np
 import ta
 import numbers
 import plotly.graph_objects as go
+from collections import defaultdict
 
 from azapy.util.drawdown import drawdown, max_drawdown
 
@@ -49,7 +50,7 @@ class Port_Simple:
         sdate : date like, optional
             Start date for historical data. If set to `None` the `sdate` will
             be set to the earliest date in `mktdata`. The default is `None`.
-        edate : date like, optional
+        edate : date-like, optional
             End date for historical dates and so the simulation. Must be
             greater than `sdate`. If it is `None` then `edate` will be set
             to the latest date in `mktdata`. The default is `None`.
@@ -116,6 +117,8 @@ class Port_Simple:
 
         self.ww = None
         self.port = None
+        
+        self.options = None
 
     def set_model(self, ww=None):
         """
@@ -123,7 +126,7 @@ class Port_Simple:
 
         Parameters
         ----------
-        ww : list (also numpy.array or pandas.Series), optional
+        ww : `list` (also `numpy.array` or `pandas.Series`), optional
             List of weights. If it is pandas.Series the index should match
             the basket `symb`. Otherwise the weights are considered in the 
             `symb` order. If it is set to `None` than `ww` will be set 
@@ -182,32 +185,41 @@ class Port_Simple:
         """
         return self.mktdata.copy()
 
-    def port_view(self, emas=[30, 200], bollinger=False, fancy=False,
-                  saveto=None):
+    def port_view(self, emas=[30, 200], bollinger=False, **opt):
         """
         Plot the portfolio time series together with optional technical
         indicators.
 
         Parameters
         ----------
-        emas : list of int, optional
+        `emas` : `list` of int, optional
             List of EMA durations. The default is [30, 200].
-        bollinger : Boolean, optional
-            If set `True` it adds the Bollinger bands. The default is `False`.
-        fancy : Boolean, optional
-            `False` : it uses the matplotlib capabilities.
-
-            `True` : it uses plotly library for interactive time-series view.
-
-            The default is `False`.
-        saveto : str, optional
-            The name of the file where to save the plot. The default is `None`.
-
+        `bollinger` : Boolean, optional
+            If set `True` it adds the Bollinger bands. The default is `False`.    
+        `opt` : other parameters
+            * `fancy` : Boolean, optional
+                - `False` : it uses the matplotlib capabilities.
+                - `True` : it uses plotly library for interactive time-series view.
+    
+                The default is `False`.
+            * `title` : `str`, optional; plot title. The default is `'Port performance'`.
+            * `xlabel` : `str`, optional; name of x-axis. The default is `'date'`.
+            * `ylabel` : `srt`; optional; name of y-axis. The default is `None`.
+            * `saveto` : `str`, optional
+                The name of the file where to save the plot. The default is `None`.
         Returns
         -------
-        pandas.DataFrame
+        `pandas.DataFrame`
             Contains the time-series included in plot.
         """
+        options = defaultdict(lambda: None, 
+                              {'title': "Port performance",
+                               'xlabel': "date", 
+                               'fancy': False})
+        options.update(opt)
+        fancy = options['fancy']
+        saveto = options['saveto']
+
         df = self.port.copy()
 
         if isinstance(emas, list):
@@ -224,53 +236,68 @@ class Port_Simple:
             df['B_lower'] = idx.bollinger_lband()
 
         if fancy:
-            fig = self._view_plotly(df)
+            fig = self._view_plotly(df, options)
             if saveto is not None:
                 fig.write_image(saveto)
         else:
             if saveto is None:
-                df.plot()
+                df.plot(title=options['title'], 
+                        xlabel=options['xlabel'], 
+                        ylabel=options['ylabel'])
             else:
-                df.plot().get_figure().savefig(saveto)
+                df.plot(title=options['title'], 
+                        xlabel=options['xlabel'], 
+                        ylabel=options['ylabel']).get_figure().savefig(saveto)
 
         return df
 
-    def port_view_all(self, sdate=None, edate=None, componly=False,
-                      fancy=False, saveto=None):
+    def port_view_all(self, sdate=None, edate=None, componly=False, **opt):
         """
         Plot the portfolio and its component time-series in a relative bases.
 
         Parameters
         ----------
-        sdate : date like, optional
+        `sdate` : date-like, optional;
             Start date of plotted time-series. If it is set to `None`
             then the `sdate` is set to the earliest date in the time-series.
             The default is `None`.
-        edate : date like, optional
+        `edate` : date-like, optional;
             End date of plotted time-series. If it set to `None` then 
             the `edate
             is set to the most recent date of the time-series.
             The default is `None`.
-        componly : Boolean, optional
+        `componly` : Boolean, optional;
             `True` : only the portfolio components time-series are plotted.
 
             `False`: the portfolio and its components times-series are plotted.
 
             The default is `True`.
-        fancy : Boolean, optional
-            `False` : it uses the pandas plot (matplotlib) capabilities.
+        `opt` : other parameters
+            * `fancy` : Boolean, optional;
+                - `False` : it uses the pandas plot (matplotlib) capabilities.
+                - `True` : it uses plotly library for interactive time-series view.
 
-            `True` : it uses plotly library for interactive time-series view.
-
-            The default is `False`.
-        saveto : str, optional
-            The name of the file where to save the plot. The default is `None`.
+                The default is `False`.
+            * `title` : `str`, optional; plot title. The default is `'Relative performance'`.
+            * `xlabel` : `str`, optional; name of x-axis. The default is `'date'`.
+            * `ylabel` : `srt`; optional; name of y-axis. The default is `None`.
+            * `saveto` : `str`, optional;
+                The name of the file where to save the plot. The default is `None`.
 
         Returns
         -------
-        pandas.DataFrame
+        `pandas.DataFrame`
             A Data Frame containing the time-series.
         """
+        options = defaultdict(lambda: None, 
+                              {'title': "Relative perfromance",
+                               'xlable': "date", 
+                               'fancy': False})
+        options.update(opt)
+        fancy = options['fancy']
+        saveto = options['saveto']
+        
+        
         if sdate is None:
             sdate = self.sdate
         if edate is None:
@@ -286,14 +313,18 @@ class Port_Simple:
         df = df.apply(lambda x: x / x[0])
 
         if fancy:
-            fig = self._view_plotly(df)
+            fig = self._view_plotly(df, options)
             if saveto is not None:
                 fig.write_image(saveto)
         else:
             if saveto is None:
-                df.plot()
+                df.plot(title=options['title'], 
+                        xlabel=options['xlabel'], 
+                        ylabel=options['ylabel'])
             else:
-                df.plot().get_figure().savefig(saveto)
+                df.plot(title=options['title'], 
+                        xlabel=options['xlabel'], 
+                        ylabel=options['ylabel']).get_figure().savefig(saveto)
 
         return df
 
@@ -488,7 +519,7 @@ class Port_Simple:
         return res.style.format("{:.2%}").applymap(_color_negative_red)
 
 
-    def _view_plotly(self, df):
+    def _view_plotly(self, df, options):
         # Create figure
         fig = go.Figure()
 
@@ -498,7 +529,9 @@ class Port_Simple:
 
         # Set title
         fig.update_layout(
-            title_text="Port performance"
+            title_text=options['title'],
+            xaxis_title=options['xlabel'],
+            yaxis_title=options['ylabel']
         )
 
         # Add range slider
